import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;  // List, etc. comment it out if you really want to know the rest
/**
 * Creates the world and some form factor variables. Also "plants" the tree and grows it.
 * 
 * @author Phoenix Johnson
 * @version 1.2
 */
public class Background extends World
{
    int timesGrown = 0;   //keeps track of how many times it's branched to stop it before it stops greenfoot.
    int angleL = 45;   //the angle that the left branch comes off at
    int angleR = 45;   //the angle that the right branch comes off at
    double scaleFactor = 0.66666667;   //the amount that the branches shrink each growth
    List<Branch> branches = new ArrayList<Branch>(1);   //to-be a list of branches
    public Background()
    {
        super(1200, 900, 1);
        prepare();
    }
    public Background(int angleL, int angleR, double scaleFactor) {   //creates a new world with new form factor values
        super(1200, 900, 1);
        this.angleL = angleL;
        this.angleR = angleR;
        this.scaleFactor = scaleFactor;
        prepare();
    }
    public void prepare() {
        Vector trunkBase = new Vector(270, 300);   //creating a vector (line) with a upwards direction (270) and a length of 300
        Point tbp = new Point(getWidth()/2, getHeight() - 1);   //creating a point to store the starting point of the trunk of the tree
        Branch trunk = new Branch(tbp, trunkBase, scaleFactor);   //creating the trunk using the starting point, vector (line), and scale factor to multiply the vector (line) length by each iteration
        branches.add(trunk);   //adding the trunk to a list of branches
        addObject(trunk, 0,0);   //adding the trunk to the world
        addObject(new WateringCan(),0,0);   //adding the watering can to the world
        addObject(new FormFactorButton(), 1065, 865);   //adds a new form factor button to the world
    }
    public void grow() {   //generates a right and a left branch for each branch in the world
        if (timesGrown<12) {   //makes sure it wont break your computer
            for (int i=branches.size()-1; i>=0;i--) {   //goes backwards through the list of branches. new branches are added on the end of a list, so if you were to go forwards through the list, you would never get to the end because its constantly adding branches
                Branch rBranch = branches.get(i).branchR(angleR);   //tells the branch in the position in the list that the loop is on to make a right branch at an angle
                Branch lBranch = branches.get(i).branchL(angleL);   //tells the branch in the position in the list that the loop is on to make a left branch at an angle
                branches.add(rBranch);   //add those branches to the list
                branches.add(lBranch);   //same
            }
            timesGrown++;   //increase the number of times grown
        } else {   //if it will break your computer to grow it again...
            GreenfootImage b = getBackground();   //get the background of the world...
            b.setColor(Color.RED);   //set the pen's color to red...
            b.drawString("Cannot grow tree without breaking Greenfoot", getWidth()/2, getHeight()/2);   //write that you can't grow the tree any more on the image...
            Greenfoot.stop();   //and stop the simulation
        }
    }
}
