import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A branch of the tree. Made up of a starting point (the ending point of the last branch
 * except the trunk), a vector (line, that goes from the starting point and then a distance
 * in a given direction. allows easy conversions between polar and cartesian corrdinates.), 
 * and a scale factor (to reduce the length of the next branch/vector) 
 * 
 * @author Phoenix Johnson
 * @version 1.1
 */
public class Branch extends Actor
{
    double scaleFactor;   //creates the scale factor variable
    Vector startv, endv;   //creates a vector for the vector of this branch and the vector for the next branches that start at this branch
    Point startp, endp;   //creates points for the starting point and ending point of this branch
    GreenfootImage img;   // create an image to draw to new branch on
    public Branch(Point startp, Vector startv, double scaleFactor) {
        this.scaleFactor = scaleFactor;   //the stored scale factor is the one given to it
        this.startp = startp;   //same but with starting point
        this.startv = startv;   //same but with the vector
        endp = new Point(startp.x + (int)startv.dx, startp.y + (int)startv.dy);   //calculates where the vector (line) ends and stores it to the ending point
    }
    protected void addedToWorld(World w) {   //code to be ran when this object is added to the world.
        GreenfootImage img = w.getBackground();   //gets the background of the world it was added to and stores it to that image
        img.setColor(Color.WHITE);   //sets the color of the lines
        img.drawLine(startp.x, startp.y, endp.x, endp.y);   //draws a line from the start point to the end point
        img.drawImage(getImage(),startp.x, startp.y);   //actually not sure what this does or if i need it. it may be leftover from testing
        w.setBackground(img);   //sets the background to the image with the line on it
        //setImage(img);   //just for testing
    }
    public Branch branchR(int angle) {   //the function that makes the right branch
        this.endv = new Vector(startv.getDirection()+angle, startv.getLength()*scaleFactor);   //calculates the new branch's vector (line) by adding the angle and shortening it by the scale factor
        Branch r = new Branch(endp, endv, scaleFactor);   //creates a branch at this branches ending point with the vector (line) and gives it the scale factor
        getWorld().addObject(r, endp.x, endp.y);   //adds that branch to the world
        return r;   //returns the branch. Used to add the branch to the world's list of branches
    }
    public Branch branchL(int angle) {   //same as the branchR but subtracts the left angle instead of adding it
        this.endv = new Vector(startv.getDirection()-angle, startv.getLength()*scaleFactor);
        Branch l = new Branch(endp, endv, scaleFactor);
        getWorld().addObject(l, endp.x, endp.y);
        return l;
    }
}
